/*
 * Copyright (c) 2021, Texas Instruments Incorporated
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  ============ ti_msp_dl_config.c =============
 *  Configured MSPM0 DriverLib module definitions
 *
 *  DO NOT EDIT - This file is generated for the MSPM0G350X
 *  by the SysConfig tool.
 */

#include "ti_msp_dl_config.h"

DL_TimerA_backupConfig gPWM_0Backup;
DL_SPI_backupConfig gGEN_SPI_1Backup;

/*
 *  ======== SYSCFG_DL_init ========
 *  Perform any initialization needed before using any board APIs
 */
SYSCONFIG_WEAK void SYSCFG_DL_init(void)
{
    SYSCFG_DL_initPower();
    SYSCFG_DL_GPIO_init();
    /* Module-Specific Initializations*/
    SYSCFG_DL_SYSCTL_init();
    SYSCFG_DL_PWM_0_init();
    SYSCFG_DL_CAPTURE_0_init();
    SYSCFG_DL_GEN_SPI_1_init();
    SYSCFG_DL_GEN_ADC_CHAN_0_init();
    SYSCFG_DL_GEN_ADC_CHAN_1_init();
    SYSCFG_DL_VREF_init();
    SYSCFG_DL_SYSCTL_CLK_init();
    /* Ensure backup structures have no valid state */
	gPWM_0Backup.backupRdy 	= false;

	gGEN_SPI_1Backup.backupRdy 	= false;

}
/*
 * User should take care to save and restore register configuration in application.
 * See Retention Configuration section for more details.
 */
SYSCONFIG_WEAK bool SYSCFG_DL_saveConfiguration(void)
{
    bool retStatus = true;

	retStatus &= DL_TimerA_saveConfiguration(PWM_0_INST, &gPWM_0Backup);
	retStatus &= DL_SPI_saveConfiguration(GEN_SPI_1_INST, &gGEN_SPI_1Backup);

    return retStatus;
}


SYSCONFIG_WEAK bool SYSCFG_DL_restoreConfiguration(void)
{
    bool retStatus = true;

	retStatus &= DL_TimerA_restoreConfiguration(PWM_0_INST, &gPWM_0Backup, false);
	retStatus &= DL_SPI_restoreConfiguration(GEN_SPI_1_INST, &gGEN_SPI_1Backup);

    return retStatus;
}

SYSCONFIG_WEAK void SYSCFG_DL_initPower(void)
{
    DL_GPIO_reset(GPIOA);
    DL_GPIO_reset(GPIOB);
    DL_TimerA_reset(PWM_0_INST);
    DL_TimerG_reset(CAPTURE_0_INST);
    DL_SPI_reset(GEN_SPI_1_INST);
    DL_ADC12_reset(GEN_ADC_CHAN_0_INST);
    DL_ADC12_reset(GEN_ADC_CHAN_1_INST);
    DL_VREF_reset(VREF);

    DL_GPIO_enablePower(GPIOA);
    DL_GPIO_enablePower(GPIOB);
    DL_TimerA_enablePower(PWM_0_INST);
    DL_TimerG_enablePower(CAPTURE_0_INST);
    DL_SPI_enablePower(GEN_SPI_1_INST);
    DL_ADC12_enablePower(GEN_ADC_CHAN_0_INST);
    DL_ADC12_enablePower(GEN_ADC_CHAN_1_INST);
    DL_VREF_enablePower(VREF);
    delay_cycles(POWER_STARTUP_DELAY);
}

SYSCONFIG_WEAK void SYSCFG_DL_GPIO_init(void)
{

    DL_GPIO_initPeripheralOutputFunction(GPIO_PWM_0_C1_IOMUX,GPIO_PWM_0_C1_IOMUX_FUNC);
    DL_GPIO_enableOutput(GPIO_PWM_0_C1_PORT, GPIO_PWM_0_C1_PIN);
    DL_GPIO_initPeripheralOutputFunction(GPIO_PWM_0_C1_CMPL_IOMUX,GPIO_PWM_0_C1_CMPL_IOMUX_FUNC);
    DL_GPIO_enableOutput(GPIO_PWM_0_C1_CMPL_PORT, GPIO_PWM_0_C1_CMPL_PIN);
    DL_GPIO_initPeripheralOutputFunction(GPIO_PWM_0_C2_IOMUX,GPIO_PWM_0_C2_IOMUX_FUNC);
    DL_GPIO_enableOutput(GPIO_PWM_0_C2_PORT, GPIO_PWM_0_C2_PIN);
    DL_GPIO_initPeripheralOutputFunction(GPIO_PWM_0_C2_CMPL_IOMUX,GPIO_PWM_0_C2_CMPL_IOMUX_FUNC);
    DL_GPIO_enableOutput(GPIO_PWM_0_C2_CMPL_PORT, GPIO_PWM_0_C2_CMPL_PIN);
    DL_GPIO_initPeripheralOutputFunction(GPIO_PWM_0_C3_IOMUX,GPIO_PWM_0_C3_IOMUX_FUNC);
    DL_GPIO_enableOutput(GPIO_PWM_0_C3_PORT, GPIO_PWM_0_C3_PIN);
    DL_GPIO_initPeripheralOutputFunction(GPIO_PWM_0_C3_CMPL_IOMUX,GPIO_PWM_0_C3_CMPL_IOMUX_FUNC);
    DL_GPIO_enableOutput(GPIO_PWM_0_C3_CMPL_PORT, GPIO_PWM_0_C3_CMPL_PIN);
    DL_GPIO_initPeripheralInputFunction(GPIO_PWM_0_IOMUX_FAULT_2,GPIO_PWM_0_IOMUX_FAULT_2_FUNC);

    DL_GPIO_initPeripheralOutputFunction(
        GPIO_GEN_SPI_1_IOMUX_SCLK, GPIO_GEN_SPI_1_IOMUX_SCLK_FUNC);
    DL_GPIO_initPeripheralOutputFunction(
        GPIO_GEN_SPI_1_IOMUX_PICO, GPIO_GEN_SPI_1_IOMUX_PICO_FUNC);
    DL_GPIO_initPeripheralInputFunction(
        GPIO_GEN_SPI_1_IOMUX_POCI, GPIO_GEN_SPI_1_IOMUX_POCI_FUNC);
    DL_GPIO_initPeripheralOutputFunction(
        GPIO_GEN_SPI_1_IOMUX_CS2, GPIO_GEN_SPI_1_IOMUX_CS2_FUNC);

    DL_GPIO_initDigitalOutput(GEN_GPIO_OUT_PINO_02_IOMUX);

    DL_GPIO_initDigitalOutput(GEN_GPIO_OUT_PINO_01_IOMUX);

    DL_GPIO_initDigitalInput(GEN_GPIO_IN_PINI_01_IOMUX);

    DL_GPIO_initDigitalInput(GEN_GPIO_IN_PINI_02_IOMUX);

    DL_GPIO_initDigitalInput(GEN_GPIO_IN_PINI_03_IOMUX);

    DL_GPIO_clearPins(GEN_GPIO_OUT_PORT, GEN_GPIO_OUT_PINO_02_PIN |
		GEN_GPIO_OUT_PINO_01_PIN);
    DL_GPIO_enableOutput(GEN_GPIO_OUT_PORT, GEN_GPIO_OUT_PINO_02_PIN |
		GEN_GPIO_OUT_PINO_01_PIN);
    DL_GPIO_setLowerPinsPolarity(GEN_GPIO_IN_PORT, DL_GPIO_PIN_12_EDGE_RISE_FALL |
		DL_GPIO_PIN_6_EDGE_RISE_FALL);
    DL_GPIO_setUpperPinsPolarity(GEN_GPIO_IN_PORT, DL_GPIO_PIN_17_EDGE_RISE_FALL);
    DL_GPIO_clearInterruptStatus(GEN_GPIO_IN_PORT, GEN_GPIO_IN_PINI_01_PIN |
		GEN_GPIO_IN_PINI_02_PIN |
		GEN_GPIO_IN_PINI_03_PIN);
    DL_GPIO_enableInterrupt(GEN_GPIO_IN_PORT, GEN_GPIO_IN_PINI_01_PIN |
		GEN_GPIO_IN_PINI_02_PIN |
		GEN_GPIO_IN_PINI_03_PIN);
    DL_GPIO_setPublisherChanID(GEN_GPIO_IN_PORT, DL_GPIO_PUBLISHER_INDEX_0, GPIOB_EVENT_PUBLISHER_0_CHANNEL);
    DL_GPIO_enableEvents(GEN_GPIO_IN_PORT, DL_GPIO_EVENT_ROUTE_1, GEN_GPIO_IN_PINI_01_PIN);

}


static const DL_SYSCTL_SYSPLLConfig gSYSPLLConfig = {
    .inputFreq              = DL_SYSCTL_SYSPLL_INPUT_FREQ_16_32_MHZ,
	.rDivClk2x              = 0,
	.rDivClk1               = 0,
	.rDivClk0               = 0,
	.enableCLK2x            = DL_SYSCTL_SYSPLL_CLK2X_DISABLE,
	.enableCLK1             = DL_SYSCTL_SYSPLL_CLK1_DISABLE,
	.enableCLK0             = DL_SYSCTL_SYSPLL_CLK0_ENABLE,
	.sysPLLMCLK             = DL_SYSCTL_SYSPLL_MCLK_CLK0,
	.sysPLLRef              = DL_SYSCTL_SYSPLL_REF_SYSOSC,
	.qDiv                   = 9,
	.pDiv                   = DL_SYSCTL_SYSPLL_PDIV_2,
	
};
SYSCONFIG_WEAK void SYSCFG_DL_SYSCTL_init(void)
{
    DL_SYSCTL_setSYSOSCFreq(DL_SYSCTL_SYSOSC_FREQ_BASE);

    /* Check that SYSPLL is disabled before configuration */
    while ((DL_SYSCTL_getClockStatus() & (DL_SYSCTL_CLK_STATUS_SYSPLL_OFF))
           != (DL_SYSCTL_CLK_STATUS_SYSPLL_OFF))
        {
            ;
        }
    DL_SYSCTL_configSYSPLL((DL_SYSCTL_SYSPLLConfig *) &gSYSPLLConfig);
	
    DL_SYSCTL_setMCLKSource(SYSOSC, HSCLK, DL_SYSCTL_HSCLK_SOURCE_SYSPLL);
    DL_SYSCTL_setMCLKDivider(DL_SYSCTL_MCLK_DIVIDER_DISABLE);
    DL_SYSCTL_setULPCLKDivider(DL_SYSCTL_ULPCLK_DIV_1);

	//Low Power Mode is configured to be SLEEP0
    DL_SYSCTL_setBORThreshold(DL_SYSCTL_BOR_THRESHOLD_LEVEL_0);
    DL_SYSCTL_setFlashWaitState(DL_SYSCTL_FLASH_WAIT_STATE_2);

}
SYSCONFIG_WEAK void SYSCFG_DL_SYSCTL_CLK_init(void) {
    while ((DL_SYSCTL_getClockStatus() & (DL_SYSCTL_CLK_STATUS_SYSPLL_GOOD
		 | DL_SYSCTL_CLK_STATUS_HSCLK_GOOD
		 | DL_SYSCTL_CLK_STATUS_LFOSC_GOOD))
	       != (DL_SYSCTL_CLK_STATUS_SYSPLL_GOOD
		 | DL_SYSCTL_CLK_STATUS_HSCLK_GOOD
		 | DL_SYSCTL_CLK_STATUS_LFOSC_GOOD))
	{
		;
	}
}



/*
 * Timer clock configuration to be sourced by  / 1 (80000000 Hz)
 * timerClkFreq = (timerClkSrc / (timerClkDivRatio * (timerClkPrescale + 1)))
 *   80000000 Hz = 80000000 Hz / (1 * (0 + 1))
 */
static const DL_TimerA_ClockConfig gPWM_0ClockConfig = {
    .clockSel = DL_TIMER_CLOCK_BUSCLK,
    .divideRatio = DL_TIMER_CLOCK_DIVIDE_1,
    .prescale = 0U
};

static const DL_TimerA_PWMConfig gPWM_0Config = {
    .pwmMode = DL_TIMER_PWM_MODE_CENTER_ALIGN,
    .period = 4000,
    .isTimerWithFourCC = true,
    .startTimer = DL_TIMER_START,
};

SYSCONFIG_WEAK void SYSCFG_DL_PWM_0_init(void) {

    DL_TimerA_setClockConfig(
        PWM_0_INST, (DL_TimerA_ClockConfig *) &gPWM_0ClockConfig);

    DL_TimerA_initPWMMode(
        PWM_0_INST, (DL_TimerA_PWMConfig *) &gPWM_0Config);

    DL_TimerA_setCaptureCompareValue(PWM_0_INST, 1000, DL_TIMER_CC_1_INDEX);
    DL_TimerA_setCaptureCompareOutCtl(PWM_0_INST, DL_TIMER_CC_OCTL_INIT_VAL_LOW,
		DL_TIMER_CC_OCTL_INV_OUT_DISABLED, DL_TIMER_CC_OCTL_SRC_DEAD_BAND,
		DL_TIMERA_CAPTURE_COMPARE_1_INDEX);

    DL_TimerA_setCaptCompUpdateMethod(PWM_0_INST, DL_TIMER_CC_UPDATE_METHOD_IMMEDIATE, DL_TIMERA_CAPTURE_COMPARE_1_INDEX);

    DL_TimerA_setCaptureCompareValue(PWM_0_INST, 1000, DL_TIMER_CC_2_INDEX);
    DL_TimerA_setCaptureCompareOutCtl(PWM_0_INST, DL_TIMER_CC_OCTL_INIT_VAL_LOW,
		DL_TIMER_CC_OCTL_INV_OUT_DISABLED, DL_TIMER_CC_OCTL_SRC_DEAD_BAND,
		DL_TIMERA_CAPTURE_COMPARE_2_INDEX);

    DL_TimerA_setCaptCompUpdateMethod(PWM_0_INST, DL_TIMER_CC_UPDATE_METHOD_IMMEDIATE, DL_TIMERA_CAPTURE_COMPARE_2_INDEX);

    DL_TimerA_setCaptureCompareValue(PWM_0_INST, 1000, DL_TIMER_CC_3_INDEX);
    DL_TimerA_setCaptureCompareOutCtl(PWM_0_INST, DL_TIMER_CC_OCTL_INIT_VAL_LOW,
		DL_TIMER_CC_OCTL_INV_OUT_DISABLED, DL_TIMER_CC_OCTL_SRC_DEAD_BAND,
		DL_TIMERA_CAPTURE_COMPARE_3_INDEX);

    DL_TimerA_setCaptCompUpdateMethod(PWM_0_INST, DL_TIMER_CC_UPDATE_METHOD_IMMEDIATE, DL_TIMERA_CAPTURE_COMPARE_3_INDEX);

    DL_TimerA_setDeadBand(PWM_0_INST, 80, 80, DL_TIMER_DEAD_BAND_MODE_0);
    DL_TimerA_enableClock(PWM_0_INST);
    DL_TimerA_enableEvent(PWM_0_INST, DL_TIMER_EVENT_ROUTE_1, (DL_TIMER_EVENT_LOAD_EVENT));

    DL_TimerA_setPublisherChanID(PWM_0_INST, DL_TIMER_PUBLISHER_INDEX_0, PWM_0_INST_PUB_0_CH);

    DL_TimerA_enableEvent(PWM_0_INST, DL_TIMER_EVENT_ROUTE_2, (DL_TIMER_EVENT_LOAD_EVENT));

    DL_TimerA_setPublisherChanID(PWM_0_INST, DL_TIMER_PUBLISHER_INDEX_1, PWM_0_INST_PUB_1_CH);

    DL_TimerA_enableInterrupt(PWM_0_INST , DL_TIMER_INTERRUPT_ZERO_EVENT);

    DL_TimerA_setCCPDirection(PWM_0_INST , DL_TIMER_CC1_OUTPUT | DL_TIMER_CC2_OUTPUT | DL_TIMER_CC3_OUTPUT );

    DL_TimerA_setFaultSourceConfig(PWM_0_INST, (DL_TIMERA_FAULT_SOURCE_EXTERNAL_2_SENSE_LOW));
    DL_TimerA_setFaultConfig(PWM_0_INST, DL_TIMERA_FAULT_CONFIG_TFIM_DISABLED
		 | DL_TIMERA_FAULT_CONFIG_FL_LATCH_SW_CLR
		 | DL_TIMERA_FAULT_CONFIG_FI_DEPENDENT
		 | DL_TIMERA_FAULT_CONFIG_FIEN_ENABLED);
    DL_TimerA_setFaultInputFilterConfig(PWM_0_INST,
        DL_TIMERA_FAULT_FILTER_FILTERED,
        DL_TIMERA_FAULT_FILTER_CPV_CONSEC_PER,
        DL_TIMERA_FAULT_FILTER_FP_PER_8);
    DL_TimerA_configFaultOutputAction(PWM_0_INST,
        DL_TIMERA_FAULT_ENTRY_CCP_LOW,
        DL_TIMERA_FAULT_EXIT_CCP_LOW,
        DL_TIMER_CC_1_INDEX);
    DL_TimerA_configFaultOutputAction(PWM_0_INST,
        DL_TIMERA_FAULT_ENTRY_CCP_LOW,
        DL_TIMERA_FAULT_EXIT_CCP_LOW,
        DL_TIMER_CC_2_INDEX);
    DL_TimerA_configFaultOutputAction(PWM_0_INST,
        DL_TIMERA_FAULT_ENTRY_CCP_LOW,
        DL_TIMERA_FAULT_EXIT_CCP_LOW,
        DL_TIMER_CC_3_INDEX);
    DL_TimerA_configFaultCounter(PWM_0_INST,
        DL_TIMERA_FAULT_ENTRY_CTR_SUSP_COUNT, DL_TIMERA_FAULT_EXIT_CTR_CVAE_ACTION);
    DL_TimerA_enableFaultInput(PWM_0_INST);
    DL_TimerA_disableClockFaultDetection(PWM_0_INST);
}



/*
 * Timer clock configuration to be sourced by BUSCLK /  (10000000 Hz)
 * timerClkFreq = (timerClkSrc / (timerClkDivRatio * (timerClkPrescale + 1)))
 *   100000 Hz = 10000000 Hz / (8 * (99 + 1))
 */
static const DL_TimerG_ClockConfig gCAPTURE_0ClockConfig = {
    .clockSel    = DL_TIMER_CLOCK_BUSCLK,
    .divideRatio = DL_TIMER_CLOCK_DIVIDE_8,
    .prescale = 99U
};

/*
 * Timer load value (where the counter starts from) is calculated as (timerPeriod * timerClockFreq) - 1
 * CAPTURE_0_INST_LOAD_VALUE = (0.5 * 100000 Hz) - 1
 */
static const DL_TimerG_CaptureTriggerConfig gCAPTURE_0CaptureConfig = {
    .captureMode    = DL_TIMER_CAPTURE_MODE_EDGE_TIME,
    .period         = CAPTURE_0_INST_LOAD_VALUE,
    .startTimer     = DL_TIMER_START,
};

SYSCONFIG_WEAK void SYSCFG_DL_CAPTURE_0_init(void) {

    DL_TimerG_setClockConfig(CAPTURE_0_INST,
        (DL_TimerG_ClockConfig *) &gCAPTURE_0ClockConfig);

    DL_TimerG_initCaptureTriggerMode(CAPTURE_0_INST,
        (DL_TimerG_CaptureTriggerConfig *) &gCAPTURE_0CaptureConfig);
    DL_TimerG_enableInterrupt(CAPTURE_0_INST , DL_TIMERG_INTERRUPT_CC0_DN_EVENT);

    DL_TimerG_enableClock(CAPTURE_0_INST);

    DL_TimerG_setExternalTriggerEvent(CAPTURE_0_INST,
        DL_TIMER_EXT_TRIG_SEL_TRIG_SUB_0);

    DL_TimerG_enableExternalTrigger(CAPTURE_0_INST);

    DL_TimerG_setSubscriberChanID(CAPTURE_0_INST,
        DL_TIMER_SUBSCRIBER_INDEX_0, CAPTURE_0_INST_SUB_0_CH);
}

static const DL_SPI_Config gGEN_SPI_1_config = {
    .mode        = DL_SPI_MODE_CONTROLLER,
    .frameFormat = DL_SPI_FRAME_FORMAT_MOTO4_POL0_PHA1,
    .parity      = DL_SPI_PARITY_NONE,
    .dataSize    = DL_SPI_DATA_SIZE_16,
    .bitOrder    = DL_SPI_BIT_ORDER_MSB_FIRST,
    .chipSelectPin = DL_SPI_CHIP_SELECT_2,
};

static const DL_SPI_ClockConfig gGEN_SPI_1_clockConfig = {
    .clockSel    = DL_SPI_CLOCK_BUSCLK,
    .divideRatio = DL_SPI_CLOCK_DIVIDE_RATIO_1
};

SYSCONFIG_WEAK void SYSCFG_DL_GEN_SPI_1_init(void) {
    DL_SPI_setClockConfig(GEN_SPI_1_INST, (DL_SPI_ClockConfig *) &gGEN_SPI_1_clockConfig);

    DL_SPI_init(GEN_SPI_1_INST, (DL_SPI_Config *) &gGEN_SPI_1_config);

    /* Configure Controller mode */
    /*
     * Set the bit rate clock divider to generate the serial output clock
     *     outputBitRate = (spiInputClock) / ((1 + SCR) * 2)
     *     1000000 = (80000000)/((1 + 39) * 2)
     */
    DL_SPI_setBitRateSerialClockDivider(GEN_SPI_1_INST, 39);
    /* Set RX and TX FIFO threshold levels */
    DL_SPI_setFIFOThreshold(GEN_SPI_1_INST, DL_SPI_RX_FIFO_LEVEL_FULL, DL_SPI_TX_FIFO_LEVEL_EMPTY);

    /* Enable module */
    DL_SPI_enable(GEN_SPI_1_INST);
}

/* GEN_ADC_CHAN_0 Initialization */
static const DL_ADC12_ClockConfig gGEN_ADC_CHAN_0ClockConfig = {
    .clockSel       = DL_ADC12_CLOCK_SYSOSC,
    .divideRatio    = DL_ADC12_CLOCK_DIVIDE_1,
    .freqRange      = DL_ADC12_CLOCK_FREQ_RANGE_24_TO_32,
};
SYSCONFIG_WEAK void SYSCFG_DL_GEN_ADC_CHAN_0_init(void)
{
    DL_ADC12_setClockConfig(GEN_ADC_CHAN_0_INST, (DL_ADC12_ClockConfig *) &gGEN_ADC_CHAN_0ClockConfig);

    DL_ADC12_initSeqSample(GEN_ADC_CHAN_0_INST,
        DL_ADC12_REPEAT_MODE_ENABLED, DL_ADC12_SAMPLING_SOURCE_AUTO, DL_ADC12_TRIG_SRC_EVENT,
        DL_ADC12_SEQ_START_ADDR_00, DL_ADC12_SEQ_END_ADDR_02, DL_ADC12_SAMP_CONV_RES_12_BIT,
        DL_ADC12_SAMP_CONV_DATA_FORMAT_UNSIGNED);
    DL_ADC12_configConversionMem(GEN_ADC_CHAN_0_INST, GEN_ADC_CHAN_0_ADCMEM_0,
        DL_ADC12_INPUT_CHAN_2, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED,
        DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_TRIGGER_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
    DL_ADC12_configConversionMem(GEN_ADC_CHAN_0_INST, GEN_ADC_CHAN_0_ADCMEM_1,
        DL_ADC12_INPUT_CHAN_3, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED,
        DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
    DL_ADC12_configConversionMem(GEN_ADC_CHAN_0_INST, GEN_ADC_CHAN_0_ADCMEM_2,
        DL_ADC12_INPUT_CHAN_7, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED,
        DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
    DL_ADC12_setSampleTime0(GEN_ADC_CHAN_0_INST,32);
    DL_ADC12_setSampleTime1(GEN_ADC_CHAN_0_INST,32);
    DL_ADC12_setSubscriberChanID(GEN_ADC_CHAN_0_INST,GEN_ADC_CHAN_0_INST_SUB_CH);
    /* Enable ADC12 interrupt */
    DL_ADC12_clearInterruptStatus(GEN_ADC_CHAN_0_INST,(DL_ADC12_INTERRUPT_MEM2_RESULT_LOADED));
    DL_ADC12_enableInterrupt(GEN_ADC_CHAN_0_INST,(DL_ADC12_INTERRUPT_MEM2_RESULT_LOADED));
    DL_ADC12_enableConversions(GEN_ADC_CHAN_0_INST);
}
/* GEN_ADC_CHAN_1 Initialization */
static const DL_ADC12_ClockConfig gGEN_ADC_CHAN_1ClockConfig = {
    .clockSel       = DL_ADC12_CLOCK_SYSOSC,
    .divideRatio    = DL_ADC12_CLOCK_DIVIDE_1,
    .freqRange      = DL_ADC12_CLOCK_FREQ_RANGE_24_TO_32,
};
SYSCONFIG_WEAK void SYSCFG_DL_GEN_ADC_CHAN_1_init(void)
{
    DL_ADC12_setClockConfig(GEN_ADC_CHAN_1_INST, (DL_ADC12_ClockConfig *) &gGEN_ADC_CHAN_1ClockConfig);

    DL_ADC12_initSeqSample(GEN_ADC_CHAN_1_INST,
        DL_ADC12_REPEAT_MODE_ENABLED, DL_ADC12_SAMPLING_SOURCE_AUTO, DL_ADC12_TRIG_SRC_EVENT,
        DL_ADC12_SEQ_START_ADDR_00, DL_ADC12_SEQ_END_ADDR_03, DL_ADC12_SAMP_CONV_RES_12_BIT,
        DL_ADC12_SAMP_CONV_DATA_FORMAT_UNSIGNED);
    DL_ADC12_configConversionMem(GEN_ADC_CHAN_1_INST, GEN_ADC_CHAN_1_ADCMEM_0,
        DL_ADC12_INPUT_CHAN_2, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED,
        DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_TRIGGER_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
    DL_ADC12_configConversionMem(GEN_ADC_CHAN_1_INST, GEN_ADC_CHAN_1_ADCMEM_1,
        DL_ADC12_INPUT_CHAN_3, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED,
        DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
    DL_ADC12_configConversionMem(GEN_ADC_CHAN_1_INST, GEN_ADC_CHAN_1_ADCMEM_2,
        DL_ADC12_INPUT_CHAN_5, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED,
        DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
    DL_ADC12_configConversionMem(GEN_ADC_CHAN_1_INST, GEN_ADC_CHAN_1_ADCMEM_3,
        DL_ADC12_INPUT_CHAN_6, DL_ADC12_REFERENCE_VOLTAGE_VDDA, DL_ADC12_SAMPLE_TIMER_SOURCE_SCOMP0, DL_ADC12_AVERAGING_MODE_DISABLED,
        DL_ADC12_BURN_OUT_SOURCE_DISABLED, DL_ADC12_TRIGGER_MODE_AUTO_NEXT, DL_ADC12_WINDOWS_COMP_MODE_DISABLED);
    DL_ADC12_setSampleTime0(GEN_ADC_CHAN_1_INST,32);
    DL_ADC12_setSampleTime1(GEN_ADC_CHAN_1_INST,32);
    DL_ADC12_setSubscriberChanID(GEN_ADC_CHAN_1_INST,GEN_ADC_CHAN_1_INST_SUB_CH);
    /* Enable ADC12 interrupt */
    DL_ADC12_clearInterruptStatus(GEN_ADC_CHAN_1_INST,(DL_ADC12_INTERRUPT_MEM3_RESULT_LOADED));
    DL_ADC12_enableInterrupt(GEN_ADC_CHAN_1_INST,(DL_ADC12_INTERRUPT_MEM3_RESULT_LOADED));
    DL_ADC12_enableConversions(GEN_ADC_CHAN_1_INST);
}


static const DL_VREF_ClockConfig gVREFClockConfig = {
    .clockSel = DL_VREF_CLOCK_BUSCLK,
    .divideRatio = DL_VREF_CLOCK_DIVIDE_1,
};
static const DL_VREF_Config gVREFConfig = {
    .vrefEnable     = DL_VREF_ENABLE_ENABLE,
    .bufConfig      = DL_VREF_BUFCONFIG_OUTPUT_2_5V,
    .shModeEnable   = DL_VREF_SHMODE_DISABLE,
    .holdCycleCount = DL_VREF_HOLD_MIN,
    .shCycleCount   = DL_VREF_SH_MIN,
};

SYSCONFIG_WEAK void SYSCFG_DL_VREF_init(void) {
    DL_VREF_setClockConfig(VREF,
        (DL_VREF_ClockConfig *) &gVREFClockConfig);
    DL_VREF_configReference(VREF,
        (DL_VREF_Config *) &gVREFConfig);
}


