/*
 * Copyright (c) 2021, Texas Instruments Incorporated - http://www.ti.com
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * *  Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * *  Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * *  Neither the name of Texas Instruments Incorporated nor the names of
 *    its contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 *  ============ ti_msp_dl_config.h =============
 *  Configured MSPM0 DriverLib module declarations
 *
 *  DO NOT EDIT - This file is generated for the MSPM0G350X
 *  by the SysConfig tool.
 */
#ifndef ti_msp_dl_config_h
#define ti_msp_dl_config_h

#define CONFIG_MSPM0G350X

#if defined(__ti_version__) || defined(__TI_COMPILER_VERSION__)
#define SYSCONFIG_WEAK __attribute__((weak))
#elif defined(__IAR_SYSTEMS_ICC__)
#define SYSCONFIG_WEAK __weak
#elif defined(__GNUC__)
#define SYSCONFIG_WEAK __attribute__((weak))
#endif

#include <ti/devices/msp/msp.h>
#include <ti/driverlib/driverlib.h>
#include <ti/driverlib/m0p/dl_core.h>

#ifdef __cplusplus
extern "C" {
#endif

/*
 *  ======== SYSCFG_DL_init ========
 *  Perform all required MSP DL initialization
 *
 *  This function should be called once at a point before any use of
 *  MSP DL.
 */


/* clang-format off */

#define POWER_STARTUP_DELAY                                                (16)


#define CPUCLK_FREQ                                                     80000000



/* Defines for PWM_0 */
#define PWM_0_INST                                                         TIMA0
#define PWM_0_INST_IRQHandler                                   TIMA0_IRQHandler
#define PWM_0_INST_INT_IRQN                                     (TIMA0_INT_IRQn)
#define PWM_0_INST_CLK_FREQ                                             80000000
/* GPIO defines for channel 1 */
#define GPIO_PWM_0_C1_PORT                                                 GPIOB
#define GPIO_PWM_0_C1_PIN                                         DL_GPIO_PIN_20
#define GPIO_PWM_0_C1_IOMUX                                      (IOMUX_PINCM48)
#define GPIO_PWM_0_C1_IOMUX_FUNC                     IOMUX_PINCM48_PF_TIMA0_CCP1
#define GPIO_PWM_0_C1_IDX                                    DL_TIMER_CC_1_INDEX
/* GPIO defines for channel 1 */
#define GPIO_PWM_0_C1_CMPL_PORT                                            GPIOB
#define GPIO_PWM_0_C1_CMPL_PIN                                    DL_GPIO_PIN_13
#define GPIO_PWM_0_C1_CMPL_IOMUX                                 (IOMUX_PINCM30)
#define GPIO_PWM_0_C1_CMPL_IOMUX_FUNC           IOMUX_PINCM30_PF_TIMA0_CCP1_CMPL

/* GPIO defines for channel 2 */
#define GPIO_PWM_0_C2_PORT                                                 GPIOB
#define GPIO_PWM_0_C2_PIN                                          DL_GPIO_PIN_4
#define GPIO_PWM_0_C2_IOMUX                                      (IOMUX_PINCM17)
#define GPIO_PWM_0_C2_IOMUX_FUNC                     IOMUX_PINCM17_PF_TIMA0_CCP2
#define GPIO_PWM_0_C2_IDX                                    DL_TIMER_CC_2_INDEX
/* GPIO defines for channel 2 */
#define GPIO_PWM_0_C2_CMPL_PORT                                            GPIOB
#define GPIO_PWM_0_C2_CMPL_PIN                                     DL_GPIO_PIN_1
#define GPIO_PWM_0_C2_CMPL_IOMUX                                 (IOMUX_PINCM13)
#define GPIO_PWM_0_C2_CMPL_IOMUX_FUNC           IOMUX_PINCM13_PF_TIMA0_CCP2_CMPL

/* GPIO defines for channel 3 */
#define GPIO_PWM_0_C3_PORT                                                 GPIOA
#define GPIO_PWM_0_C3_PIN                                         DL_GPIO_PIN_28
#define GPIO_PWM_0_C3_IOMUX                                       (IOMUX_PINCM3)
#define GPIO_PWM_0_C3_IOMUX_FUNC                      IOMUX_PINCM3_PF_TIMA0_CCP3
#define GPIO_PWM_0_C3_IDX                                    DL_TIMER_CC_3_INDEX
/* GPIO defines for channel 3 */
#define GPIO_PWM_0_C3_CMPL_PORT                                            GPIOA
#define GPIO_PWM_0_C3_CMPL_PIN                                    DL_GPIO_PIN_31
#define GPIO_PWM_0_C3_CMPL_IOMUX                                  (IOMUX_PINCM6)
#define GPIO_PWM_0_C3_CMPL_IOMUX_FUNC            IOMUX_PINCM6_PF_TIMA0_CCP3_CMPL

#define GPIO_PWM_0_IOMUX_FAULT_2                                 (IOMUX_PINCM60)
#define GPIO_PWM_0_IOMUX_FAULT_2_FUNC               IOMUX_PINCM60_PF_TIMA_FAULT2

/* Publisher defines */
#define PWM_0_INST_PUB_0_CH                                                  (1)
#define PWM_0_INST_PUB_1_CH                                                  (2)


/* Defines for CAPTURE_0 */
#define CAPTURE_0_INST                                                   (TIMG0)
#define CAPTURE_0_INST_IRQHandler                               TIMG0_IRQHandler
#define CAPTURE_0_INST_INT_IRQN                                 (TIMG0_INT_IRQn)
#define CAPTURE_0_INST_LOAD_VALUE                                       (49999U)
#define CAPTURE_0_INST_SUB_0_CH                                              (3)





/* Defines for GEN_SPI_1 */
#define GEN_SPI_1_INST                                                     SPI1
#define GEN_SPI_1_INST_IRQHandler                               SPI1_IRQHandler
#define GEN_SPI_1_INST_INT_IRQN                                   SPI1_INT_IRQn
#define GPIO_GEN_SPI_1_PICO_PORT                                          GPIOB
#define GPIO_GEN_SPI_1_PICO_PIN                                   DL_GPIO_PIN_8
#define GPIO_GEN_SPI_1_IOMUX_PICO                               (IOMUX_PINCM25)
#define GPIO_GEN_SPI_1_IOMUX_PICO_FUNC               IOMUX_PINCM25_PF_SPI1_PICO
#define GPIO_GEN_SPI_1_POCI_PORT                                          GPIOB
#define GPIO_GEN_SPI_1_POCI_PIN                                   DL_GPIO_PIN_7
#define GPIO_GEN_SPI_1_IOMUX_POCI                               (IOMUX_PINCM24)
#define GPIO_GEN_SPI_1_IOMUX_POCI_FUNC               IOMUX_PINCM24_PF_SPI1_POCI
/* GPIO configuration for GEN_SPI_1 */
#define GPIO_GEN_SPI_1_SCLK_PORT                                          GPIOB
#define GPIO_GEN_SPI_1_SCLK_PIN                                   DL_GPIO_PIN_9
#define GPIO_GEN_SPI_1_IOMUX_SCLK                               (IOMUX_PINCM26)
#define GPIO_GEN_SPI_1_IOMUX_SCLK_FUNC               IOMUX_PINCM26_PF_SPI1_SCLK
#define GPIO_GEN_SPI_1_CS2_PORT                                           GPIOB
#define GPIO_GEN_SPI_1_CS2_PIN                                    DL_GPIO_PIN_0
#define GPIO_GEN_SPI_1_IOMUX_CS2                                (IOMUX_PINCM12)
#define GPIO_GEN_SPI_1_IOMUX_CS2_FUNC           IOMUX_PINCM12_PF_SPI1_CS2_POCI2



/* Defines for GEN_ADC_CHAN_0 */
#define GEN_ADC_CHAN_0_INST                                                 ADC0
#define GEN_ADC_CHAN_0_INST_IRQHandler                           ADC0_IRQHandler
#define GEN_ADC_CHAN_0_INST_INT_IRQN                             (ADC0_INT_IRQn)
#define GEN_ADC_CHAN_0_ADCMEM_0                               DL_ADC12_MEM_IDX_0
#define GEN_ADC_CHAN_0_ADCMEM_0_REF              DL_ADC12_REFERENCE_VOLTAGE_VDDA
#define GEN_ADC_CHAN_0_ADCMEM_0_REF_VOLTAGE                                      -1 // VDDA cannot be determined
#define GEN_ADC_CHAN_0_ADCMEM_1                               DL_ADC12_MEM_IDX_1
#define GEN_ADC_CHAN_0_ADCMEM_1_REF              DL_ADC12_REFERENCE_VOLTAGE_VDDA
#define GEN_ADC_CHAN_0_ADCMEM_1_REF_VOLTAGE                                      -1 // VDDA cannot be determined
#define GEN_ADC_CHAN_0_ADCMEM_2                               DL_ADC12_MEM_IDX_2
#define GEN_ADC_CHAN_0_ADCMEM_2_REF              DL_ADC12_REFERENCE_VOLTAGE_VDDA
#define GEN_ADC_CHAN_0_ADCMEM_2_REF_VOLTAGE                                      -1 // VDDA cannot be determined
#define GEN_ADC_CHAN_0_INST_SUB_CH                                           (1)
#define GPIO_GEN_ADC_CHAN_0_C2_PORT                                        GPIOA
#define GPIO_GEN_ADC_CHAN_0_C2_PIN                                DL_GPIO_PIN_25
#define GPIO_GEN_ADC_CHAN_0_C3_PORT                                        GPIOA
#define GPIO_GEN_ADC_CHAN_0_C3_PIN                                DL_GPIO_PIN_24
#define GPIO_GEN_ADC_CHAN_0_C7_PORT                                        GPIOA
#define GPIO_GEN_ADC_CHAN_0_C7_PIN                                DL_GPIO_PIN_22

/* Defines for GEN_ADC_CHAN_1 */
#define GEN_ADC_CHAN_1_INST                                                 ADC1
#define GEN_ADC_CHAN_1_INST_IRQHandler                           ADC1_IRQHandler
#define GEN_ADC_CHAN_1_INST_INT_IRQN                             (ADC1_INT_IRQn)
#define GEN_ADC_CHAN_1_ADCMEM_0                               DL_ADC12_MEM_IDX_0
#define GEN_ADC_CHAN_1_ADCMEM_0_REF              DL_ADC12_REFERENCE_VOLTAGE_VDDA
#define GEN_ADC_CHAN_1_ADCMEM_0_REF_VOLTAGE                                      -1 // VDDA cannot be determined
#define GEN_ADC_CHAN_1_ADCMEM_1                               DL_ADC12_MEM_IDX_1
#define GEN_ADC_CHAN_1_ADCMEM_1_REF              DL_ADC12_REFERENCE_VOLTAGE_VDDA
#define GEN_ADC_CHAN_1_ADCMEM_1_REF_VOLTAGE                                      -1 // VDDA cannot be determined
#define GEN_ADC_CHAN_1_ADCMEM_2                               DL_ADC12_MEM_IDX_2
#define GEN_ADC_CHAN_1_ADCMEM_2_REF              DL_ADC12_REFERENCE_VOLTAGE_VDDA
#define GEN_ADC_CHAN_1_ADCMEM_2_REF_VOLTAGE                                      -1 // VDDA cannot be determined
#define GEN_ADC_CHAN_1_ADCMEM_3                               DL_ADC12_MEM_IDX_3
#define GEN_ADC_CHAN_1_ADCMEM_3_REF              DL_ADC12_REFERENCE_VOLTAGE_VDDA
#define GEN_ADC_CHAN_1_ADCMEM_3_REF_VOLTAGE                                      -1 // VDDA cannot be determined
#define GEN_ADC_CHAN_1_INST_SUB_CH                                           (2)
#define GPIO_GEN_ADC_CHAN_1_C2_PORT                                        GPIOA
#define GPIO_GEN_ADC_CHAN_1_C2_PIN                                DL_GPIO_PIN_17
#define GPIO_GEN_ADC_CHAN_1_C3_PORT                                        GPIOA
#define GPIO_GEN_ADC_CHAN_1_C3_PIN                                DL_GPIO_PIN_18
#define GPIO_GEN_ADC_CHAN_1_C5_PORT                                        GPIOB
#define GPIO_GEN_ADC_CHAN_1_C5_PIN                                DL_GPIO_PIN_18
#define GPIO_GEN_ADC_CHAN_1_C6_PORT                                        GPIOB
#define GPIO_GEN_ADC_CHAN_1_C6_PIN                                DL_GPIO_PIN_19


/* Defines for VREF */
#define VREF_VOLTAGE_MV                                                     2500
#define GPIO_VREF_VREFPOS_PORT                                             GPIOA
#define GPIO_VREF_VREFPOS_PIN                                     DL_GPIO_PIN_23
#define GPIO_VREF_IOMUX_VREFPOS                                  (IOMUX_PINCM53)
#define GPIO_VREF_IOMUX_VREFPOS_FUNC                IOMUX_PINCM53_PF_UNCONNECTED




/* Port definition for Pin Group GEN_GPIO_OUT */
#define GEN_GPIO_OUT_PORT                                                (GPIOA)

/* Defines for PINO_02: GPIOA.26 with pinCMx 59 on package pin 30 */
#define GEN_GPIO_OUT_PINO_02_PIN                                (DL_GPIO_PIN_26)
#define GEN_GPIO_OUT_PINO_02_IOMUX                               (IOMUX_PINCM59)
/* Defines for PINO_01: GPIOA.11 with pinCMx 22 on package pin 57 */
#define GEN_GPIO_OUT_PINO_01_PIN                                (DL_GPIO_PIN_11)
#define GEN_GPIO_OUT_PINO_01_IOMUX                               (IOMUX_PINCM22)
/* Port definition for Pin Group GEN_GPIO_IN */
#define GEN_GPIO_IN_PORT                                                 (GPIOB)

/* Defines for PINI_01: GPIOB.12 with pinCMx 29 on package pin 64 */
// pins affected by this interrupt request:["PINI_01","PINI_02","PINI_03"]
#define GEN_GPIO_IN_INT_IRQN                                    (GPIOB_INT_IRQn)
#define GEN_GPIO_IN_INT_IIDX                    (DL_INTERRUPT_GROUP1_IIDX_GPIOB)
#define GEN_GPIO_IN_PINI_01_IIDX                            (DL_GPIO_IIDX_DIO12)
#define GEN_GPIO_IN_PINI_01_PIN                                 (DL_GPIO_PIN_12)
#define GEN_GPIO_IN_PINI_01_IOMUX                                (IOMUX_PINCM29)
/* Defines for PINI_02: GPIOB.17 with pinCMx 43 on package pin 14 */
#define GEN_GPIO_IN_PINI_02_IIDX                            (DL_GPIO_IIDX_DIO17)
#define GEN_GPIO_IN_PINI_02_PIN                                 (DL_GPIO_PIN_17)
#define GEN_GPIO_IN_PINI_02_IOMUX                                (IOMUX_PINCM43)
/* Defines for PINI_03: GPIOB.6 with pinCMx 23 on package pin 58 */
#define GEN_GPIO_IN_PINI_03_IIDX                             (DL_GPIO_IIDX_DIO6)
#define GEN_GPIO_IN_PINI_03_PIN                                  (DL_GPIO_PIN_6)
#define GEN_GPIO_IN_PINI_03_IOMUX                                (IOMUX_PINCM23)
#define GPIOB_EVENT_PUBLISHER_0_CHANNEL                                      (3)

#define GPIO_FAULT_1_FAULT_PIN_2_PORT                                      GPIOA
#define GPIO_FAULT_1_FAULT_PIN_2                                  DL_GPIO_PIN_27
#define GPIO_FAULT_1_IOMUX_FAULT_PIN_2                           (IOMUX_PINCM60)
#define GPIO_FAULT_1_IOMUX_FAULT_PIN_2_FUNC            IOMUX_PINCM60_PF_TIMA_FAULT2

/* clang-format on */

void SYSCFG_DL_init(void);
void SYSCFG_DL_initPower(void);
void SYSCFG_DL_GPIO_init(void);
void SYSCFG_DL_SYSCTL_init(void);
void SYSCFG_DL_SYSCTL_CLK_init(void);
void SYSCFG_DL_PWM_0_init(void);
void SYSCFG_DL_CAPTURE_0_init(void);
void SYSCFG_DL_GEN_SPI_1_init(void);
void SYSCFG_DL_GEN_ADC_CHAN_0_init(void);
void SYSCFG_DL_GEN_ADC_CHAN_1_init(void);
void SYSCFG_DL_VREF_init(void);


bool SYSCFG_DL_saveConfiguration(void);
bool SYSCFG_DL_restoreConfiguration(void);

#ifdef __cplusplus
}
#endif

#endif /* ti_msp_dl_config_h */
